/*
Bin_Button.cc - binary button for use with the edanator

Copyright (C) 2004  Edward S. Arthur (ed.arthur@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <iostream>

#include "edanator.h"
#include "Bin_Button.h"
#include "Bin_Hex_Widget.h"

extern edanator_opts opts;

const char *bit_string[64] = {
  "0",  "1",  "2",  "3",  "4",  "5",  "6",  "7",  "8",  "9",
  "10", "11", "12", "13", "14", "15", "16", "17", "18", "19",
  "20", "21", "22", "23", "24", "25", "26", "27", "28", "29",
  "30", "31", "32", "33", "34", "35", "36", "37", "38", "39",
  "40", "41", "42", "43", "44", "45", "46", "47", "48", "49",
  "50", "51", "52", "53", "54", "55", "56", "57", "58", "59",
  "60", "61", "62", "63"};


const char *bit_string_too_low = "<?";
const char *bit_string_too_hi  = ">?";


Bin_Button::Bin_Button(int x, int y, int width, int height, char *label, int bit_select, int endian_val) :
  Fl_Button(x, y+BIT_NUMBER_HEIGHT, width, height, label) {
  this->x      = x;
  this->y      = y;
  this->value  = 0;
  this->endian = endian_val;

  bit_box     = new Fl_Box(x, y, BIN_BUTTON_WIDTH, BIT_NUMBER_HEIGHT,
			     this->get_string(bit_select));
  bit_box->labelsize(7);
  bit_box->box(FL_NO_BOX);
}

int Bin_Button::get_value(void)
{
  return this->value;
}

void Bin_Button::toggle_bit(void)
{
  Bin_Hex_Widget *p;

  this->set_value(this->value ? 0 : 1);
    
  p = (Bin_Hex_Widget *) this->parent();
  p->fix_nibble(1);
}

int Bin_Button::handle(int event)
{
  Bin_Hex_Widget *parent = (Bin_Hex_Widget *) this->parent();

  switch (event) {
  case FL_KEYDOWN: {
    int ascii = Fl::event_text()[0]; // get key pressed

    switch (ascii) {
    case '0': {
      this->set_value(0);
      parent->fix_nibble(1);
      return 1;
    }
    case '1': {
      this->set_value(1);
      parent->fix_nibble(1);
      return 1;
    }
    case ' ': {
      toggle_bit();
      return 1;
    }
    }
  }
  case FL_ENTER: {
    this->take_focus();
    return 1;
  }
  }

  return Fl_Button::handle(event); // standard event handler
}

void Bin_Button::update_bit(void)
{
  this->color(this->value ? FL_BLACK : FL_GRAY);
  this->redraw();
}


void Bin_Button::set_value(int value)
{
  this->value = value;
  this->update_bit();
}


void Bin_Button::cb_key_press_bin(Fl_Widget *w, void *p)
{
  Bin_Button *bb = (Bin_Button *) p;
  bb->toggle_bit();
}


const char *Bin_Button::get_string(int bit_index)
{
  // big endian      n-1, n-2, .. 1, 0
  // little endian   0, 1, .. n-2, n-1

  if (bit_index < 0) {
    return bit_string_too_low;
  }
  else if (bit_index >= 64) {
    return bit_string_too_hi;
  }
  else {
    if (endian == EDANATOR_BIG_ENDIAN) {
      return bit_string[bit_index];
    }
    else {
      return bit_string[opts.bits - 1 - bit_index];
    }
  }
}
