/*
Bin_Hex_Array.cc - binary hex array for use with the edanator

Copyright (C) 2004  Edward S. Arthur (ed.arthur@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <stdio.h>
#include <iostream>

#include "edanator.h"
#include "FL/Fl.H"
#include "Bin_Button.h"
#include "Hex_Button.h"
#include "Bin_Hex_Widget.h"
#include "Bin_Hex_Array.h"

extern edanator_opts opts;

Bin_Hex_Array::Bin_Hex_Array(int x, int y, int width, int height, const char *label, int endian_val) :
    Fl_Group(x, y, width, height, label) {

  int i, x_val = x;
  
  endian = endian_val;
  bh = new Bin_Hex_Widget*[opts.nibs];

  for (i = opts.nibs-1; i >= 0; i--) {
    // set up bin_hex buttons
    
    bh[i] = new Bin_Hex_Widget(x_val, y,
			       BIN_HEX_WIDGET_WIDTH, BIN_HEX_WIDGET_HEIGHT,
			       NULL, i, endian);

    x_val += BIN_HEX_WIDGET_WIDTH;

    // add space on 4 nibble boundaries
    if ((i % 4) == 0) {
      x_val += 5;
    }
  }

  left4 = new Fl_Button(x_val + 10, y + BIT_NUMBER_HEIGHT, 15, 15, "@<<");
  left4->callback(this->shift_left_nib_cb, this);

  left1 = new Fl_Button(x_val + 30, y + BIT_NUMBER_HEIGHT, 15, 15, "@<");
  left1->callback(this->shift_left_bit_cb, this);

  clear = new Fl_Button(x_val + 50, y + BIT_NUMBER_HEIGHT, 15, 15, "C");
  clear->callback(this->clear_cb, this);

  right1 = new Fl_Button(x_val + 70, y + BIT_NUMBER_HEIGHT, 15, 15, "@>");
  right1->callback(this->shift_right_bit_cb, this);

  right4 = new Fl_Button(x_val + 90, y + BIT_NUMBER_HEIGHT, 15, 15, "@>>");
  right4->callback(this->shift_right_nib_cb, this);

  endianness = new Fl_Button(x_val + 110, y + BIT_NUMBER_HEIGHT, 15, 15,
			     endian_label());
  endianness->callback(this->swap_endianness_cb, this);

  // input field
  input = new Fl_Int_Input(x_val + 10, y + 20 + BIT_NUMBER_HEIGHT, 150, 15, NULL);
  input->callback(this->input_cb, this);
  input->when(FL_WHEN_ENTER_KEY|FL_WHEN_NOT_CHANGED);
  input->value(INPUT_PROMPT);

  // output fields
  hex_out = new Fl_Output(x_val + 10, y + 40 + BIT_NUMBER_HEIGHT, 150, 15, NULL);
  dec_out = new Fl_Output(x_val + 10, y + 60 + BIT_NUMBER_HEIGHT, 150, 15, NULL);

  // end of the group

  end();
} 


const char *Bin_Hex_Array::endian_label(void)
{
  // returns the opposite of what we're set to
  // so the user gets the endianness they want
  return (endian == EDANATOR_BIG_ENDIAN) ? "e" : "E";
}

void Bin_Hex_Array::show(void) {
  int i;

  for (i = 0; i < opts.nibs; i++) {
    bh[i]->show();
  }

  left1->show();
  left4->show();

  right1->show();
  right4->show();
  endianness->show();

  clear->show();

  input->show();
  
  hex_out->show();
  dec_out->show();
}


void Bin_Hex_Array::shift_left_bit_cb(Fl_Widget *w, void *p)
{
  Bin_Hex_Array *bha = (Bin_Hex_Array *) p;
  int i;
  int val, prev_val = 0;

  for (i = 0; i < opts.nibs; i++) {
    val = bha->bh[i]->bb[3]->get_value();
    bha->bh[i]->bb[3]->set_value(bha->bh[i]->bb[2]->get_value());
    bha->bh[i]->bb[2]->set_value(bha->bh[i]->bb[1]->get_value());
    bha->bh[i]->bb[1]->set_value(bha->bh[i]->bb[0]->get_value());
    bha->bh[i]->bb[0]->set_value(prev_val);
    prev_val = val;
    bha->bh[i]->fix_nibble(0);
    bha->bh[i]->fix_bits(1);
  }
}


void Bin_Hex_Array::shift_right_bit_cb(Fl_Widget *w, void *p)
{
  Bin_Hex_Array *bha = (Bin_Hex_Array *) p;
  int i;
  int val, prev_val = 0;

  for (i = opts.nibs-1; i >= 0; i--) {
    val = bha->bh[i]->bb[0]->get_value();
    bha->bh[i]->bb[0]->set_value(bha->bh[i]->bb[1]->get_value());
    bha->bh[i]->bb[1]->set_value(bha->bh[i]->bb[2]->get_value());
    bha->bh[i]->bb[2]->set_value(bha->bh[i]->bb[3]->get_value());
    bha->bh[i]->bb[3]->set_value(prev_val);
    prev_val = val;
    bha->bh[i]->fix_nibble(0);
    bha->bh[i]->fix_bits(1);
  }
}


void Bin_Hex_Array::shift_left_nib_cb(Fl_Widget *w, void *p)
{
  Bin_Hex_Array *bha = (Bin_Hex_Array *) p;
  int i;
  int val, prev_val = 0;

  for (i = 0; i < opts.nibs; i++) {
    val = bha->bh[i]->hb->get_value();
    bha->bh[i]->hb->set_value(prev_val);
    prev_val = val;
    bha->bh[i]->fix_bits(0);
    bha->bh[i]->fix_nibble(1);
  }
}


void Bin_Hex_Array::shift_right_nib_cb(Fl_Widget *w, void *p)
{
  Bin_Hex_Array *bha = (Bin_Hex_Array *) p;
  int i;
  int val, prev_val = 0;

  for (i = opts.nibs-1; i >= 0; i--) {
    val = bha->bh[i]->hb->get_value();
    bha->bh[i]->hb->set_value(prev_val);
    prev_val = val;
    bha->bh[i]->fix_bits(0);
    bha->bh[i]->fix_nibble(1);
  }
}


void Bin_Hex_Array::swap_endianness_cb(Fl_Widget *w, void *p)
{
  Bin_Hex_Array *bha = (Bin_Hex_Array *) p;
  int i, j, bit = 0;


  bha->endian = (bha->endian == EDANATOR_BIG_ENDIAN) ?
                 EDANATOR_LITTLE_ENDIAN : EDANATOR_BIG_ENDIAN;

  for (i = opts.nibs-1; i >= 0; i--) {
    for (j = 3; j >= 0; j--) {
      Bin_Button *bb = bha->bh[i]->bb[j];

      bb->bit_box->label(bb->get_string(bit));
      bb->endian = bha->endian;
      bit++;
    }
  }

  bha->endianness->label(bha->endian_label());
}



void Bin_Hex_Array::clear_cb(Fl_Widget *w, void *p)
{
  Bin_Hex_Array *bha = (Bin_Hex_Array *) p;
  int i;

  for (i = 0; i < opts.nibs; i++) {
    bha->bh[i]->hb->set_value(0);
    bha->bh[i]->fix_bits(0);
    bha->bh[i]->fix_nibble(1);
  }
}


void Bin_Hex_Array::input_cb(Fl_Widget *w, void *p)
{
  Bin_Hex_Array *bha = (Bin_Hex_Array *) p;
  unsigned long long int l, l_shifted;
  int i;
  const char *input_string = bha->input->value();

  // if the user hits return, clear prompt string and a use value of 0
  if (strcmp(input_string, INPUT_PROMPT) == 0) {
    bha->input->value("");
    l = 0ULL;
  }

  if (CASECMP(input_string, "0x", 2) == 0) {
    sscanf(input_string, IN_LL_FMT_HEX, &l);
  }
  else {
    sscanf(input_string, IN_LL_FMT_DEC, &l);
  }

  l_shifted = l;
  for (i = 0; i < opts.nibs; i++) {
    bha->bh[i]->hb->set_value((int) (l_shifted & 0xF));
    bha->bh[i]->fix_bits(0);
    bha->bh[i]->fix_nibble(1);
    l_shifted = l_shifted >> 4;
  }
}


unsigned long long int Bin_Hex_Array::get_value(void)
{
  unsigned long long int l = 0;
  int i;
  int start_nib = opts.nibs - 1;

  for (i = start_nib; i >= 0; i--) {
    l = this->bh[i]->hb->get_value() | (l << 4);
  }
  return l;
}


unsigned long long int Bin_Hex_Array::get_reverse_value(void)
{
  unsigned long long int l = 0;
  int i, j;

  for (i = 0; i < opts.nibs; i++) {
    for (j = 0; j < 4; j++) {
      l = (l << 1) | this->bh[i]->bb[j]->get_value();
    }
  }
  return l;
}



void Bin_Hex_Array::fix_output(void)
{
  char scratch[80];
  unsigned long long int l;

  l = this->get_value();

  sprintf(scratch, OUT_LL_FMT_HEX, l);
  this->hex_out->value(scratch);

  sprintf(scratch, OUT_LL_FMT_DEC, l);
  this->dec_out->value(scratch);
}


void Bin_Hex_Array::set_value(unsigned long long int value)
{
  int i, nibble;

  for (i = 0; i < opts.nibs; i++) {
    nibble = (int) value & 0xF;
    this->bh[i]->hb->set_value(nibble);
    value >>= 4;
    this->bh[i]->fix_bits(0);
    this->bh[i]->fix_nibble(1);
  }
}
