/*
Bin_Hex_Widget.cc - binary hex widget for use with the edanator

Copyright (C) 2004  Edward S. Arthur (ed.arthur@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <iostream>

#include "Bin_Button.h"
#include "Hex_Button.h"
#include "Bin_Hex_Widget.h"
#include "Bin_Hex_Array.h"


Bin_Hex_Widget::Bin_Hex_Widget(int x, int y, int width, int height, const char *label, int nibble, int endian) :
    Fl_Group(x, y, width, height, label) {

  int nib = nibble * 4;
  int my_x = x;

  // set up bin buttons

  bb[3] = new Bin_Button(my_x, y, BIN_BUTTON_WIDTH, BIN_BUTTON_WIDTH, "", nib + 3, endian);
  bb[3]->callback(Bin_Button::cb_key_press_bin, bb[3]);
  my_x += BIN_BUTTON_SPACE + BIN_BUTTON_WIDTH;

  bb[2] = new Bin_Button(my_x, y, BIN_BUTTON_WIDTH, BIN_BUTTON_WIDTH, "", nib + 2, endian);
  bb[2]->callback(Bin_Button::cb_key_press_bin, bb[2]);
  my_x += BIN_BUTTON_SPACE + BIN_BUTTON_WIDTH;

  bb[1] = new Bin_Button(my_x, y, BIN_BUTTON_WIDTH, BIN_BUTTON_WIDTH, "", nib + 1, endian);
  bb[1]->callback(Bin_Button::cb_key_press_bin, bb[1]);
  my_x += BIN_BUTTON_SPACE + BIN_BUTTON_WIDTH;

  bb[0] = new Bin_Button(my_x, y, BIN_BUTTON_WIDTH, BIN_BUTTON_WIDTH, "", nib, endian);
  bb[0]->callback(Bin_Button::cb_key_press_bin, bb[0]);

  // set up hex button

  hb = new Hex_Button(x, y + BIN_BUTTON_HEIGHT + BIT_NUMBER_HEIGHT, BIN_HEX_WIDGET_WIDTH,
		      BIN_HEX_WIDGET_WIDTH, "0");

  // end of the group

  end();
} 


void Bin_Hex_Widget::show(void) {
  bb[0]->show();
  bb[1]->show();
  bb[2]->show();
  bb[3]->show();
  hb->show();
}


void Bin_Hex_Widget::fix_nibble(int call_parent) {
  Bin_Hex_Array *parent = (Bin_Hex_Array *) this->parent();

  hb->set_value((bb[3]->get_value() << 3) | (bb[2]->get_value() << 2) |
		(bb[1]->get_value() << 1) | bb[0]->get_value());
  hb->label(hb->get_nib_str(hb->get_value()));

  if (call_parent) {
    parent->fix_output();
  }
}


void Bin_Hex_Widget::fix_bits(int call_parent) {
  Bin_Hex_Array *parent = (Bin_Hex_Array *) this->parent();

  bb[3]->set_value((hb->get_value() & 8) >> 3);
  bb[2]->set_value((hb->get_value() & 4) >> 2);
  bb[1]->set_value((hb->get_value() & 2) >> 1);
  bb[0]->set_value(hb->get_value() & 1);

  bb[3]->update_bit();
  bb[2]->update_bit();
  bb[1]->update_bit();
  bb[0]->update_bit();

  if (call_parent) {
    parent->fix_output();
  }
}
