/*
Hex_Button.cc - hex button for use with the edanator

Copyright (C) 2004  Edward S. Arthur (ed.arthur@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <iostream>

#include "Hex_Button.h"
#include "Bin_Hex_Widget.h"

static char nib_str[16][2] = {"0", "1", "2", "3", "4", "5", "6", "7",
			      "8", "9", "A", "B", "C", "D", "E", "F"};


int Hex_Button::get_value(void) {
  return this->value;
}

int Hex_Button::handle(int event) {
  Bin_Hex_Widget *parent = (Bin_Hex_Widget *) this->parent();

  switch (event) {
  case FL_KEYDOWN: {
    int ascii = Fl::event_text()[0]; // get key pressed

    if (isxdigit(ascii)) {
      if (isdigit(ascii)) {
	this->value = ascii - '0';
      }
      else if (islower(ascii)) {
	this->value = ascii - 'a' + 10;
      }
      else {
	// upper case
	this->value = ascii - 'A' + 10;
      }

      this->label(get_nib_str(this->value));
      parent->fix_bits(1);
      return 1;
    }
    else if (ascii == ' ') {
      this->increment(parent);
      return 1;
    }
  }
  case FL_ENTER: {
    this->take_focus();
    return 1;
  }
  case FL_PUSH: {
    int mouse_button = Fl::event_button();
    switch (mouse_button) {
    case FL_LEFT_MOUSE: {
      this->increment(parent);
      return 1;
    }
    case FL_RIGHT_MOUSE: {
      this->decrement(parent);
      return 1;
    }
    } // end of switch
  }
  } // end of switch
  return Fl_Button::handle(event); // standard event handler
}


char *Hex_Button::get_nib_str(int idx) {
  idx &= 0xF;

  return nib_str[idx];
}

void Hex_Button::set_value(int value)
{
  this->value = value;
}


void Hex_Button::increment(Bin_Hex_Widget *parent)
{
  if (this->value == 0xF) {
    this->value = 0;
  }
  else {
    this->value++;
  }
  this->label(get_nib_str(this->value));
  parent->fix_bits(1);
}


void Hex_Button::decrement(Bin_Hex_Widget *parent)
{
  if (this->value == 0x0) {
    this->value = 0xF;
  }
  else {
    this->value--;
  }
  this->label(get_nib_str(this->value));
  parent->fix_bits(1);
}
