/*
edanator.cc - binary/hex graphical calculator

Copyright (C) 2004  Edward S. Arthur (ed.arthur@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Menu_Item.H>
#include <FL/Fl_Window.H>
#include <FL/Fl.H>
#include <FL/fl_ask.H>

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <iostream>

#include "Bin_Button.h"
#include "Hex_Button.h"
#include "Bin_Hex_Widget.h"
#include "Bin_Hex_Array.h"
#include "edanator.h"

// globals

char *prog = "edanator";
edanator_opts opts;
Bin_Hex_Array *x, *y, *z;
Fl_Menu_Bar *m;
Fl_Window *wind;

static void z_is_x_plus_y(void);
static void z_is_x_minus_y(void);
static void z_is_x_and_y(void);
static void z_is_x_or_y(void);
static void z_is_x_xor_y(void);
static void x_is_not_x(void);
static void y_is_not_y(void);
static void z_is_not_x(void);
static void z_is_not_y(void);
static void copy_z_to_x(void);
static void copy_z_to_y(void);
static void y_is_rev_x(void);
static void z_is_rev_x(void);
static void z_is_rev_y(void);

static void hide_y(void);
static void hide_z(void);
static void show_y(void);
static void show_z(void);

static void set_bits(int bits);
static void set_bits_32(void);
static void set_bits_36(void);
static void set_bits_40(void);
static void set_bits_44(void);
static void set_bits_48(void);
static void set_bits_52(void);
static void set_bits_56(void);
static void set_bits_60(void);
static void set_bits_64(void);

static void show_help(void);

void show_menu(void);

Fl_Menu_Item menuitems_x[] = {
  { "&File",          0,          0,        0,      FL_SUBMENU},
    { "E&xit", FL_CTRL + 'q', (Fl_Callback *) exit, 0},
    { 0 },
  { "&View",          0,          0,        0,      FL_SUBMENU},
    { "Show &Y", FL_CTRL + 'y', (Fl_Callback *) show_y, 0},
    { "Show &Z", FL_CTRL + 'z', (Fl_Callback *) show_z, 0},
#if 0    
    { "Width",        0,          0,        0,      FL_SUBMENU},
       { "32-bits",   0, (Fl_Callback *) set_bits, (void *) 32 },
       { "36-bits",   0, (Fl_Callback *) set_bits, (void *) 36 },
       { "40-bits",   0, (Fl_Callback *) set_bits, (void *) 40 },
       { "44-bits",   0, (Fl_Callback *) set_bits, (void *) 44 },
       { "48-bits",   0, (Fl_Callback *) set_bits, (void *) 48 },
       { "52-bits",   0, (Fl_Callback *) set_bits, (void *) 52 },
       { "56-bits",   0, (Fl_Callback *) set_bits, (void *) 56 },
       { "60-bits",   0, (Fl_Callback *) set_bits, (void *) 60 },
       { "64-bits",   0, (Fl_Callback *) set_bits, (void *) 64 },
       { 0 },
#endif       
    { 0 },
  { "&Calculate",     0,          0,        0,      FL_SUBMENU},
    { "X = ~X",         0, (Fl_Callback *) x_is_not_x,     0},
    { 0 },
  { "&Help", FL_CTRL + 'h', (Fl_Callback *) show_help,     0},
  { 0 }
};

Fl_Menu_Item menuitems_xy[] = {
  { "&File",          0,          0,        0,      FL_SUBMENU},
    { "E&xit", FL_CTRL + 'q', (Fl_Callback *) exit, 0},
    { 0 },
  { "&View",          0,          0,        0,      FL_SUBMENU},
    { "Hide &Y", FL_CTRL + 'y', (Fl_Callback *) hide_y, 0},
    { "Show &Z", FL_CTRL + 'z', (Fl_Callback *) show_z, 0, FL_MENU_DIVIDER},
#if 0    
    { "Width",       0,           0,        0,      FL_SUBMENU},
       { "32-bits",  0, (Fl_Callback *) set_bits, (void *) 32 },
       { "36-bits",  0, (Fl_Callback *) set_bits, (void *) 36 },
       { "40-bits",  0, (Fl_Callback *) set_bits, (void *) 40 },
       { "44-bits",  0, (Fl_Callback *) set_bits, (void *) 44 },
       { "48-bits",  0, (Fl_Callback *) set_bits, (void *) 48 },
       { "52-bits",  0, (Fl_Callback *) set_bits, (void *) 52 },
       { "56-bits",  0, (Fl_Callback *) set_bits, (void *) 56 },
       { "60-bits",  0, (Fl_Callback *) set_bits, (void *) 60 },
       { "64-bits",  0, (Fl_Callback *) set_bits, (void *) 64 },
       { 0 },
#endif       
    { 0 },
  { "&Calculate",     0,          0,        0,      FL_SUBMENU},
    { "X = ~X",       0, (Fl_Callback *) x_is_not_x,     0},
    { "Y = ~Y",       0, (Fl_Callback *) y_is_not_y,     0},
    { "Y = rev(X)",   0, (Fl_Callback *) y_is_rev_x,     0},
    { 0 },
  { "&Help", FL_CTRL + 'h', (Fl_Callback *) show_help,     0},
  { 0 }
};


Fl_Menu_Item menuitems_xyz[] = {
  { "&File",          0,          0,        0,      FL_SUBMENU},
    { "E&xit", FL_CTRL + 'q', (Fl_Callback *) exit, 0},
    { 0 },
  { "&View",          0,          0,        0,      FL_SUBMENU},
    { "Hide &Y", FL_CTRL + 'y', (Fl_Callback *) hide_y, 0},
    { "Hide &Z", FL_CTRL + 'z', (Fl_Callback *) hide_z, 0, FL_MENU_DIVIDER},
#if 0    
    { "Width",        0,          0,        0,      FL_SUBMENU},
       { "32-bits",   0, (Fl_Callback *) set_bits, (void *) 32 },
       { "36-bits",   0, (Fl_Callback *) set_bits, (void *) 36 },
       { "40-bits",   0, (Fl_Callback *) set_bits, (void *) 40 },
       { "44-bits",   0, (Fl_Callback *) set_bits, (void *) 44 },
       { "48-bits",   0, (Fl_Callback *) set_bits, (void *) 48 },
       { "52-bits",   0, (Fl_Callback *) set_bits, (void *) 52 },
       { "56-bits",   0, (Fl_Callback *) set_bits, (void *) 56 },
       { "60-bits",   0, (Fl_Callback *) set_bits, (void *) 60 },
       { "64-bits",   0, (Fl_Callback *) set_bits, (void *) 64 },
       { 0 },
#endif       
    { 0 },
  { "&Calculate",     0,          0,        0,      FL_SUBMENU},
    { "Z = X + Y",    0, (Fl_Callback *) z_is_x_plus_y,  0},
    { "Z = X - Y",    0, (Fl_Callback *) z_is_x_minus_y, 0},
    { "Z = X | Y",    0, (Fl_Callback *) z_is_x_or_y,    0},
    { "Z = X && Y",   0, (Fl_Callback *) z_is_x_and_y,   0},
    { "Z = X ^ Y",    0, (Fl_Callback *) z_is_x_xor_y,   0},
    { "X = ~X",       0, (Fl_Callback *) x_is_not_x,     0},
    { "Y = ~Y",       0, (Fl_Callback *) y_is_not_y,     0},
    { "Z = ~X",       0, (Fl_Callback *) z_is_not_x,     0},
    { "Z = ~Y",       0, (Fl_Callback *) z_is_not_y,     0},
    { "Z -> X",       0, (Fl_Callback *) copy_z_to_x,    0},
    { "Z -> Y",       0, (Fl_Callback *) copy_z_to_y,    0},
    { "Y = rev(X)",   0, (Fl_Callback *) y_is_rev_x,     0},
    { "Z = rev(X)",   0, (Fl_Callback *) z_is_rev_x,     0},
    { "Z = rev(Y)",   0, (Fl_Callback *) z_is_rev_y,     0},
    { 0 },
  { "&Help", FL_CTRL + 'h', (Fl_Callback *) show_help,     0},    
  { 0 }
};

void parse_opts(int argc, char **argv)
{
  int c;
  int temp;

  while ((c = getopt(argc, argv, "b:eEhn:vyz")) != -1) {
    switch (c) {
    case 'b': {
      temp = atoi(optarg);
      if ((temp > 0) && (temp <= MAX_BITS)) {
	opts.bits = temp;
      }
      else {
	fprintf(stderr, "%s: Error, illegal number of bits specified (%d)\n", prog, temp);
	fprintf(stderr, "Maximum number of bits is %d\n", MAX_BITS);
	exit(1);
      }

      if ((opts.bits % 4) == 0) {
	opts.nibs = opts.bits/4;
      }
      else {
	fprintf(stderr, "%s: Error, illegal number of bits specified (%d)\n", prog, temp);
	fprintf(stderr, "You must specify a multipe of 4 (maximum number of bits is %d)\n",
		MAX_BITS);
	exit(1);
      }
      break;
    }
    case 'e': {
      opts.endian = EDANATOR_LITTLE_ENDIAN;
      break;
    }
    case 'E': {
      opts.endian = EDANATOR_BIG_ENDIAN;
      break;
    }
    case 'h': {
      printf("%% %s [-b #BITS | -n #NIBS] [-e | -E] [-y | -z]\n\n", prog);
      printf("-b #BITS   # of bits to display    (>= 4 and <= %d) must be divisible by 4\n",
	     MAX_BITS);
      printf("-e         little endian\n");
      printf("-E         big endian (default)\n");
      printf("-n #NIBS   # of nibbles to display (>= 1 and <= %d)\n", MAX_NIBS);
      printf("-y         also displays second row (Y), default is just single (X) row\n");
      printf("-z         also displays second (Y) and third (Z) rows\n\n");
      printf("-h         display this help screen\n");
      printf("-v         display version number and quit\n");
      exit(1);
    }
    case 'n': {
      temp = atoi(optarg);
      if ((temp > 0) && (temp <= MAX_NIBS)) {
	opts.nibs = temp;
      }
      else {
	fprintf(stderr, "%s: Error, illegal number of nibbles specified (%d)\n", prog, temp);
	fprintf(stderr, "Maximum number of nibbles is %d\n", MAX_NIBS);
	exit(1);
      }

      opts.bits = opts.nibs * 4;
      break;
    }
    case 'v': {
      printf("%s\n", VERSION);
      exit(1);
    }
    case 'y': {
      opts.show_y = 1;
      break;
    }
    case 'z': {
      opts.show_y = 1;
      opts.show_z = 1;
      break;
    }
    default: {
      printf("dunno what I found, exiting\n");
      exit(1);
      break;
    }
    }
  }
}


void resize(void)
{
  wind->size(opts.width(), opts.height());
  wind->show();
}

static void show_help(void)
{
  fl_message("%s\n\n%% %s [-b #BITS | -n #NIBS] [-y | -z]\n\n"
             "http://sourceforge.net/projects/edanator\n\nCopyright (C) 2004-2007  "
             "Edward S. Arthur (ed.arthur@@gmail.com)", 
             VERSION, prog);
}
 
static void z_is_x_plus_y(void)
{
  z->set_value(x->get_value() + y->get_value());
}

static void z_is_x_minus_y(void)
{
  z->set_value(x->get_value() - y->get_value());
}

static void z_is_x_and_y(void)
{
  z->set_value(x->get_value() & y->get_value());
}

static void z_is_x_or_y(void)
{
  z->set_value(x->get_value() | y->get_value());
}

static void z_is_x_xor_y(void)
{
  z->set_value(x->get_value() ^ y->get_value());
}

static void z_is_not_x(void)
{
  z->set_value(~x->get_value());
}

static void z_is_not_y(void)
{
  z->set_value(~y->get_value());
}

static void x_is_not_x(void)
{
  x->set_value(~x->get_value());
}

static void y_is_not_y(void)
{
  y->set_value(~y->get_value());
}

static void copy_z_to_x(void)
{
  x->set_value(z->get_value());
}

static void copy_z_to_y(void)
{
  y->set_value(z->get_value());
}

static void y_is_rev_x(void)
{
  y->set_value(x->get_reverse_value());
}


static void z_is_rev_x(void)
{
  z->set_value(x->get_reverse_value());
}


static void z_is_rev_y(void)
{
  z->set_value(y->get_reverse_value());
}


static void hide_y(void)
{
  opts.show_y = 0;
  opts.show_z = 0;
  
//  delete y;
//  delete z;
  show_menu();
  resize();
}

static void hide_z(void)
{
  opts.show_z = 0;
//  delete z;
  show_menu();
  resize();
}

static void show_y(void)
{
  wind->begin();
  opts.show_y = 1;
/*
  y = new Bin_Hex_Array(0,						// starting x
			  30+BIN_HEX_ARRAY_HEIGHT,			// starting y
			  opts.width(),				// total width
			  BIN_HEX_ARRAY_HEIGHT,				// height
			  NULL);					// label
*/  
  y->show();

  opts.show_z = 0;
  show_menu();
  resize();
}

static void show_z(void)
{
  wind->begin();
  if (opts.show_y == 0) {
     opts.show_y = 1;
/*
     y = new Bin_Hex_Array(0,						// starting x
			  30+BIN_HEX_ARRAY_HEIGHT,			// starting y
			  opts.width(),				// total width
			  BIN_HEX_ARRAY_HEIGHT,				// height
			  NULL);					// label
*/
     y->show();
  }

  opts.show_z = 1;
/*
  z = new Bin_Hex_Array(0,						// starting x
			  30 + (2 * BIN_HEX_ARRAY_HEIGHT),		// starting y
			  opts.width(),				// total width
			  BIN_HEX_ARRAY_HEIGHT,				// height
			  NULL);					// label
*/
  z->show();

  show_menu();
  resize();
}

void show_menu(void)
{
  m->clear();
  if (opts.show_z) {
    m->copy(menuitems_xyz);
  }
  else if (opts.show_y) {
    m->copy(menuitems_xy);
  }
  else {
    m->copy(menuitems_x);
  }
}

static void set_bits(int bits)
{
  opts.bits = bits;
  opts.nibs = bits/4;
}

static void set_bits_32(void)
{
  opts.bits = 32;
  opts.nibs = opts.bits/4;
}

static void set_bits_34(void);
static void set_bits_40(void);
static void set_bits_44(void);
static void set_bits_48(void);
static void set_bits_52(void);
static void set_bits_56(void);
static void set_bits_60(void);
static void set_bits_64(void);

/* main program */

int main(int argc, char **argv)
{
  parse_opts(argc, argv);

  wind = new Fl_Window(opts.width(), opts.height(), VERSION);
  
  m = new Fl_Menu_Bar(0, 0, opts.width(), 30);

  show_menu();

  x = new Bin_Hex_Array(0,					// starting x
			30,					// starting y
			opts.width(),				// total width
			BIN_HEX_ARRAY_HEIGHT,			// height
			NULL,					// label
			opts.endian);

  x->show();

  y = new Bin_Hex_Array(0,					// starting x
			30+BIN_HEX_ARRAY_HEIGHT,		// starting y
			opts.width(),				// total width
			BIN_HEX_ARRAY_HEIGHT,			// height
			NULL,					// label
			opts.endian);

  if (opts.show_y) {
    y->show();
  }

  
  z = new Bin_Hex_Array(0,					// starting x
			30 + (2 * BIN_HEX_ARRAY_HEIGHT),	// starting y
			opts.width(),				// total width
			BIN_HEX_ARRAY_HEIGHT,			// height
			NULL,					// label
			opts.endian);

  if (opts.show_z) {
    z->show();
  }

  m->show();

  wind->end();
  wind->show();

  return Fl::run();
}

// edanator_opts

edanator_opts::edanator_opts(void) {
  bits   = 40;
  nibs   = bits/4;
  show_x = 1;
  show_y = 0;
  show_z = 0;
  endian = EDANATOR_BIG_ENDIAN;
}

int edanator_opts::rows(void) {
  if (this->show_z) {
    return 3;
  }
  else if (this->show_y) {
    return 2;
  }
  else {
    return 1;
  }
}
  
int edanator_opts::width(void) {
  return ((this->nibs * BIN_HEX_WIDGET_WIDTH) + 180);
}
  
int edanator_opts::height(void) {
  return ((this->rows() * BIN_HEX_ARRAY_HEIGHT) + 30);
}
